/**
 * Autogenerated by Thrift
 *
 * DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
 *  @generated
 */
package com.facebook.thrift.annotation.rust_deprecated;

import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;
import java.util.Collections;
import java.util.BitSet;
import java.util.Arrays;
import com.facebook.thrift.*;
import com.facebook.thrift.annotations.*;
import com.facebook.thrift.async.*;
import com.facebook.thrift.meta_data.*;
import com.facebook.thrift.server.*;
import com.facebook.thrift.transport.*;
import com.facebook.thrift.protocol.*;

/**
 * If `anyhow_to_application_exn` is true, this allows returning an
 * `anyhow::Error` from within a Thrift server handler method, and that
 * `anyhow::Error` will be turned into an `ApplicationException`. This is
 * similar in behavior to what happens if you throw an unhandled exception type
 * in Python Thrift server or C++ Thrift server.
 * 
 * The `ApplicationException` returned will have the error code `Unknown` and
 * message `format!("{:#}", anyhow_err)`.
 * 
 * NOTE: it is generally considered bad practice to use this because it
 * eliminates the ability to match on specific error types on the client side.
 * When possible, it is recommended you always return structured error types
 * (though it is more verbose). This annotation is provided solely for
 * convenience and should not be used services where error type matching is
 * needed.
 * 
 * Example, the following Thrift:
 * 
 * ```
 * service Foo {
 *   @rust.ServiceExn{anyhow_to_application_exn = true}
 *   void bar();
 * }
 * ```
 * 
 * would allow for the following Rust server impl:
 * 
 * ```
 * #[async_trait]
 * impl Foo for FooServerImpl {
 *     async fn bar() -> Result<(), BarExn> {
 *         if some_condition {
 *             Err(anyhow::anyhow!("some error"))?
 *         }
 * 
 *         // Note you must always convert to `anyhow::Error` first for non-`anyhow::Error`
 *         // types.
 *         some_client_call.await.context("failed some_client_call")?;
 * 
 *         // you can still return a structured exn type if desired
 *         return Err(BarExn::ie(...));
 *     }
 * }
 * ```
 * 
 * You can also use this annotation on the service definition itself to have it
 * apply to all methods on the service, e.g.
 * 
 * ```
 * @rust.ServiceExn{anyhow_to_application_exn = true}
 * service Foo {
 *   // Both `bar` and `baz` will support `anyhow::Error` -> `ApplicationException`.
 *   void bar();
 *   void baz();
 * }
 * ```
 */
@SuppressWarnings({ "unused", "serial" })
public class ServiceExn implements TBase, java.io.Serializable, Cloneable {
  private static final TStruct STRUCT_DESC = new TStruct("ServiceExn");
  private static final TField ANYHOW_TO_APPLICATION_EXN_FIELD_DESC = new TField("anyhow_to_application_exn", TType.BOOL, (short)1);

  public final Boolean anyhow_to_application_exn;
  public static final int ANYHOW_TO_APPLICATION_EXN = 1;

  public ServiceExn(
      Boolean anyhow_to_application_exn) {
    this.anyhow_to_application_exn = anyhow_to_application_exn;
  }

  /**
   * Performs a deep copy on <i>other</i>.
   */
  public ServiceExn(ServiceExn other) {
    if (other.isSetAnyhow_to_application_exn()) {
      this.anyhow_to_application_exn = TBaseHelper.deepCopy(other.anyhow_to_application_exn);
    } else {
      this.anyhow_to_application_exn = null;
    }
  }

  public ServiceExn deepCopy() {
    return new ServiceExn(this);
  }

  public Boolean isAnyhow_to_application_exn() {
    return this.anyhow_to_application_exn;
  }

  // Returns true if field anyhow_to_application_exn is set (has been assigned a value) and false otherwise
  public boolean isSetAnyhow_to_application_exn() {
    return this.anyhow_to_application_exn != null;
  }

  @Override
  public boolean equals(Object _that) {
    if (_that == null)
      return false;
    if (this == _that)
      return true;
    if (!(_that instanceof ServiceExn))
      return false;
    ServiceExn that = (ServiceExn)_that;

    if (!TBaseHelper.equalsNobinary(this.isSetAnyhow_to_application_exn(), that.isSetAnyhow_to_application_exn(), this.anyhow_to_application_exn, that.anyhow_to_application_exn)) { return false; }

    return true;
  }

  @Override
  public int hashCode() {
    return Arrays.deepHashCode(new Object[] {anyhow_to_application_exn});
  }

  // This is required to satisfy the TBase interface, but can't be implemented on immutable struture.
  public void read(TProtocol iprot) throws TException {
    throw new TException("unimplemented in android immutable structure");
  }

  public static ServiceExn deserialize(TProtocol iprot) throws TException {
    Boolean tmp_anyhow_to_application_exn = null;
    TField __field;
    iprot.readStructBegin();
    while (true)
    {
      __field = iprot.readFieldBegin();
      if (__field.type == TType.STOP) {
        break;
      }
      switch (__field.id)
      {
        case ANYHOW_TO_APPLICATION_EXN:
          if (__field.type == TType.BOOL) {
            tmp_anyhow_to_application_exn = iprot.readBool();
          } else {
            TProtocolUtil.skip(iprot, __field.type);
          }
          break;
        default:
          TProtocolUtil.skip(iprot, __field.type);
          break;
      }
      iprot.readFieldEnd();
    }
    iprot.readStructEnd();

    ServiceExn _that;
    _that = new ServiceExn(
      tmp_anyhow_to_application_exn
    );
    _that.validate();
    return _that;
  }

  public void write(TProtocol oprot) throws TException {
    validate();

    oprot.writeStructBegin(STRUCT_DESC);
    if (this.anyhow_to_application_exn != null) {
      oprot.writeFieldBegin(ANYHOW_TO_APPLICATION_EXN_FIELD_DESC);
      oprot.writeBool(this.anyhow_to_application_exn);
      oprot.writeFieldEnd();
    }
    oprot.writeFieldStop();
    oprot.writeStructEnd();
  }

  @Override
  public String toString() {
    return toString(1, true);
  }

  @Override
  public String toString(int indent, boolean prettyPrint) {
    return TBaseHelper.toStringHelper(this, indent, prettyPrint);
  }

  public void validate() throws TException {
    // check for required fields
  }

}

